@@ -8,12 +8,4 @@ from . import integrators
 from .integrators import available_methods
 from . import tests
 from .utilities import tests
-from .exception_types import tests
-
-try:
-    # Pytest testing
-    from numpy._pytesttester import PytestTester
-    test = PytestTester(__name__)
-    del PytestTester
-except:
-    pass
\ No newline at end of file
+from .exception_types import tests
\ No newline at end of file
